@@ -36,6 +36,14 @@ inline double angle(double x, double y)
   return atan2(y,x);
 }
   
+// Return an angle in the interval [0,360).
+inline double principalBranch(double deg)
+{
+  deg=fmod(deg,360.0);
+  if(deg < 0) deg += 360.0; 
+  return deg;
+}
+
 } //namespace camp
 
 #endif
