@@ -22,8 +22,10 @@
 
 namespace camp {
 
-extern double T[3]; // z-component of current transform
-  
+//extern double Tx[3]; // x-component of current transform
+//extern double Ty[3]; // y-component of current transform
+extern double Tz[3]; // z-component of current transform
+
 static const double pixel=1.0; // Adaptive rendering constant.
 
 // Return one-sixth of the second derivative of the Bezier curve defined
@@ -119,6 +121,11 @@ typedef mem::vector<groupmap> groupsmap;
 class drawElement : public gc
 {
 public:
+  string KEY;
+  
+  drawElement(const string& key="") : KEY(key == "" ? processData().KEY : key)
+  {}
+  
   virtual ~drawElement() {}
   
   static pen lastpen;  
@@ -157,6 +164,8 @@ public:
 
   virtual bool islabel() {return false;}
 
+  virtual bool isnewpage() {return false;}
+  
   virtual bool islayer() {return false;}
 
   virtual bool is3D() {return false;}
@@ -230,7 +239,8 @@ public:
     copyArray4x4C(T,&t);
   }
 
-  drawElementLC(const double* t, const drawElementLC *s) : T(NULL) {
+  drawElementLC(const double* t, const drawElementLC *s) : 
+    drawElement(s->KEY), T(NULL) {
     multiplyTransform3(T,t,s->T);
   }
 
