@@ -4,6 +4,10 @@
  * Render 3D Bezier paths and surfaces.
  *****/
 
+#ifdef __CYGWIN__
+#define _POSIX_C_SOURCE 200809L
+#endif
+
 #include <stdlib.h>
 #include <fstream>
 #include <cstring>
@@ -372,7 +376,7 @@ void Export()
                                            transform(0.0,0.0,w,0.0,0.0,h),
                                            antialias);
       pic.append(Image);
-      pic.shipout(NULL,Prefix,Format,0.0,false,View);
+      pic.shipout(NULL,Prefix,Format,false,View);
       delete Image;
       delete[] data;
     } 
@@ -1442,12 +1446,12 @@ void glrender(const string& prefix, const picture *pic, const string& format,
       osmesa_initialized=true;
       init_osmesa();
     }
-  } else {
-    if(glinitialize) {
-      glinitialize=false;
-      init();
-      Fitscreen=1;
-    }
+  }
+  
+  if(glinitialize) {
+    glinitialize=false;
+    init();
+    Fitscreen=1;
   }
 
   static bool initialized=false;
