@@ -19,6 +19,10 @@
 *
 *************/
 
+#ifdef __CYGWIN__
+#define _POSIX_C_SOURCE 200809L
+#endif
+
 #include <iostream>
 #include <cstdlib>
 #include <cerrno>
@@ -36,7 +40,7 @@
 #include "settings.h"
 #include "locate.h"
 #include "interact.h"
-#include "process.h"
+#include "fileio.h"
 
 #include "stack.h"
 
@@ -47,7 +51,7 @@ using interact::interactive;
 namespace run {
 void purge();
 }
-  
+
 #ifdef PROFILE
 namespace vm {
 extern void dumpProfile();
@@ -71,11 +75,11 @@ int sigsegv_handler (void *, int emergency)
     cerr << "Stack overflow or segmentation fault: rerun with -nothreads"
          << endl;
   else
-#endif    
+#endif
     cerr << "Segmentation fault" << endl;
   abort();
 }
-#endif 
+#endif
 
 void setsignal(RETSIGTYPE (*handler)(int))
 {
@@ -103,7 +107,13 @@ void interruptHandler(int)
   em.Interrupt(true);
 }
 
-struct Args 
+bool hangup=false;
+void hangup_handler(int sig)
+{
+  hangup=true;
+}
+
+struct Args
 {
   int argc;
   char **argv;
@@ -113,7 +123,6 @@ struct Args
 void *asymain(void *A)
 {
   setsignal(signalHandler);
-  
   Args *args=(Args *) A;
   fpu_trap(trap());
 
@@ -125,12 +134,27 @@ void *asymain(void *A)
       doUnrestrictedList();
     } catch(handled_error) {
       em.statusError();
-    } 
+    }
   } else {
     int n=numArgs();
-    if(n == 0) 
-      processFile("-");
-    else
+    if(n == 0) {
+      int inpipe=intcast(settings::getSetting<Int>("inpipe"));
+      if(inpipe >= 0) {
+        Signal(SIGHUP,hangup_handler);
+        camp::openpipeout();
+        fprintf(camp::pipeout,"\n");
+        fflush(camp::pipeout);
+      }
+      while(true) {
+        processFile("-",true);
+        try {
+          setOptions(args->argc,args->argv);
+        } catch(handled_error) {
+          em.statusError();
+        }
+        if(inpipe < 0) break;
+      }
+    } else {
       for(int ind=0; ind < n; ind++) {
         processFile(string(getArg(ind)),n > 1);
         try {
@@ -138,8 +162,9 @@ void *asymain(void *A)
             setOptions(args->argc,args->argv);
         } catch(handled_error) {
           em.statusError();
-        } 
+        }
       }
+    }
   }
 
 #ifdef PROFILE
@@ -158,7 +183,7 @@ void *asymain(void *A)
   }
 #endif
 #endif
-  exit(em.processStatus() || interact::interactive ? 0 : 1);  
+  exit(em.processStatus() || interact::interactive ? 0 : 1);
 }
 
 void exitHandler(int)
@@ -166,12 +191,13 @@ void exitHandler(int)
   exit(0);
 }
 
-int main(int argc, char *argv[]) 
+int main(int argc, char *argv[])
 {
-#ifdef HAVE_LIBGSL  
+    // cout << "hello?";
+#ifdef HAVE_LIBGSL
   unsetenv("GSL_RNG_SEED");
   unsetenv("GSL_RNG_TYPE");
-#endif  
+#endif
   setsignal(signalHandler);
 
   try {
@@ -179,17 +205,17 @@ int main(int argc, char *argv[])
   } catch(handled_error) {
     em.statusError();
   }
-  
+
   Args args(argc,argv);
 #ifdef HAVE_GL
 #ifdef __APPLE__
   bool usethreads=true;
 #else
   bool usethreads=view();
-#endif  
+#endif
   gl::glthread=usethreads ? getSetting<bool>("threads") : false;
 #if HAVE_PTHREAD
-  
+
   if(gl::glthread) {
     pthread_t thread;
     try {
@@ -211,6 +237,6 @@ int main(int argc, char *argv[])
   }
 #endif
   gl::glthread=false;
-#endif  
+#endif
   asymain(&args);
 }
