@@ -11,15 +11,21 @@
 #include <vector>
 #include <stack>
 #include <map>
+#include <deque>
 #include <string>
 #include <sstream>
 
-#ifndef __GNUC_PREREQ
-#define __GNUC_PREREQ(maj, min) (0)
-#endif
-
 #ifndef NOHASH
-#if __GNUC_PREREQ(4,3) || defined(__CYGWIN__)
+
+#ifdef HAVE_UNORDERED_MAP
+
+#include <memory>
+#include <unordered_map>
+#define EXT std
+
+#else
+
+#ifdef HAVE_TR1_UNORDERED_MAP
 #include <tr1/unordered_map>
 #define EXT std::tr1
 #else
@@ -28,6 +34,9 @@
 #define unordered_map hash_map
 #define unordered_multimap hash_multimap
 #endif
+
+#endif
+
 #endif
 
 #ifdef __DECCXX_LIBCXX_RH70
@@ -116,18 +125,23 @@ namespace mem {
 
 GC_CONTAINER(list);
 GC_CONTAINER(vector);
+GC_CONTAINER(deque);
 
 template <typename T, typename Container = vector<T> >
 struct stack : public std::stack<T, Container>, public gc {
 };
 
+#define PAIR_ALLOC gc_allocator<std::pair<CONST Key,T> > /* space */
+
 #undef GC_CONTAINER
 
 #define GC_CONTAINER(KIND)                                              \
-  template <typename Key, typename T, typename Compare = std::less<Key> > \
-  struct KIND : public                                                  \
-  std::KIND<Key,T,Compare,gc_allocator<std::pair<Key,T> > >, public gc { \
-    KIND() : std::KIND<Key,T,Compare,gc_allocator<std::pair<Key,T> > > () {} \
+  template <typename Key,                                               \
+            typename T,                                                 \
+            typename Compare = std::less<Key> >                         \
+  struct KIND : public std::KIND<Key,T,Compare,PAIR_ALLOC>, public gc   \
+  {                                                                     \
+    KIND() : std::KIND<Key,T,Compare,PAIR_ALLOC> () {}                  \
   }
 
 GC_CONTAINER(map);
@@ -141,10 +155,10 @@ GC_CONTAINER(multimap);
             typename Hash = EXT::hash<Key>,                             \
             typename Eq = std::equal_to<Key> >                          \
   struct KIND : public                                                  \
-  EXT::KIND<Key,T,Hash,Eq,gc_allocator<std::pair<Key, T> > >, public gc { \
-    KIND() : EXT::KIND<Key,T,Hash,Eq,gc_allocator<std::pair<Key, T> > > () {} \
+  EXT::KIND<Key,T,Hash,Eq,PAIR_ALLOC>, public gc {                      \
+    KIND() : EXT::KIND<Key,T,Hash,Eq,PAIR_ALLOC> () {}                  \
     KIND(size_t n)                                                      \
-      : EXT::KIND<Key,T,Hash,Eq,gc_allocator<std::pair<Key, T> > > (n) {} \
+      : EXT::KIND<Key,T,Hash,Eq,PAIR_ALLOC> (n) {}                      \
   }
 
 GC_CONTAINER(unordered_map);
@@ -154,6 +168,8 @@ GC_CONTAINER(unordered_multimap);
 #undef EXT
 #endif
 
+#undef PAIR_ALLOC
+
 #ifdef USEGC
 typedef std::basic_string<char,std::char_traits<char>,
                           gc_allocator<char> > string;
@@ -165,7 +181,13 @@ typedef std::basic_ostringstream<char,std::char_traits<char>,
                                  gc_allocator<char> > ostringstream;
 typedef std::basic_stringbuf<char,std::char_traits<char>,
                              gc_allocator<char> > stringbuf;
+#if GC_TMP_VERSION_MAJOR >= 7 && GC_TMP_VERSION_MINOR > 1
+inline void compact(int x) {GC_set_dont_expand(x);}
 #else
+inline void compact(int x) {GC_dont_expand=x;}
+#endif    
+#else
+inline void compact(int x) {}
 typedef std::string string;
 typedef std::stringstream stringstream;
 typedef std::istringstream istringstream;
