@@ -145,7 +145,8 @@ public:
   friend pair unit(const pair& z)
   {
     double scale=z.length();
-    if(scale != 0.0) scale=1.0/scale;
+    if(scale == 0.0) return z;
+    scale=1.0/scale;
     return pair(z.x*scale,z.y*scale);
   }
   
@@ -159,6 +160,11 @@ public:
     return z.x*w.x+z.y*w.y;
   }
   
+  friend double cross(const pair& z, const pair& w)
+  {
+    return z.x*w.y-z.y*w.x;
+  }
+  
 // Return the principal branch of the square root (non-negative real part).
   friend pair Sqrt(const pair& z) {
     double mag=z.length();
