@@ -28,9 +28,11 @@ const double third=1.0/3.0;
 
 path nullpath;
   
+const char *nopoints="nullpath has no points";
+
 void checkEmpty(Int n) {
   if(n == 0)
-    reportError("nullpath has no points");
+    reportError(nopoints);
 }
 
 // Accurate computation of sqrt(1+x)-1.
@@ -46,8 +48,9 @@ inline pair sqrt1pxm1(pair x)
 // Solve for the real roots of the quadratic equation ax^2+bx+c=0.
 quadraticroots::quadraticroots(double a, double b, double c)
 {
-  if(a == 0.0) {
-    if(b != 0.0) {
+  // Remove roots at numerical infinity.
+  if(fabs(a) <= Fuzz*(fabs(b)+fabs(c)*Fuzz)) {
+    if(fabs(b) > Fuzz*fabs(c)) {
       distinct=quadraticroots::ONE;
       roots=1;
       t1=-c/b;
@@ -59,39 +62,42 @@ quadraticroots::quadraticroots(double a, double b, double c)
       distinct=quadraticroots::NONE;
       roots=0;
     }
-  } else if(b == 0.0) {
-    double x=-c/a;
-    if(x >= 0.0) {
-      distinct=quadraticroots::TWO;
-      roots=2;
-      t2=sqrt(x);
-      t1=-t2;
-    } else {
-      distinct=quadraticroots::NONE;
-      roots=0;
-    }
   } else {
     double factor=0.5*b/a;
-    double x=-2.0*c/(b*factor);
-    if(x > -1.0) {
-      distinct=quadraticroots::TWO;
-      roots=2;
-      double r2=factor*sqrt1pxm1(x);
-      double r1=-r2-2.0*factor;
-      if(r1 <= r2) {
-        t1=r1;
-        t2=r2;
+    double denom=b*factor;
+    if(fabs(denom) <= Fuzz*fabs(c)) {
+      double x=-c/a;
+      if(x >= 0.0) {
+        distinct=quadraticroots::TWO;
+        roots=2;
+        t2=sqrt(x);
+        t1=-t2;
       } else {
-        t1=r2;
-        t2=r1;
+        distinct=quadraticroots::NONE;
+        roots=0;
       }
-    } else if(x == -1.0) {
-      distinct=quadraticroots::ONE;
-      roots=2;
-      t1=t2=-factor;
     } else {
-      distinct=quadraticroots::NONE;
-      roots=0;
+      double x=-2.0*c/denom;
+      if(x > -1.0) {
+        distinct=quadraticroots::TWO;
+        roots=2;
+        double r2=factor*sqrt1pxm1(x);
+        double r1=-r2-2.0*factor;
+        if(r1 <= r2) {
+          t1=r1;
+          t2=r2;
+        } else {
+          t1=r2;
+          t2=r1;
+        }
+      } else if(x == -1.0) {
+        distinct=quadraticroots::ONE;
+        roots=2;
+        t1=t2=-factor;
+      } else {
+        distinct=quadraticroots::NONE;
+        roots=0;
+      }
     }
   }
 }
@@ -110,12 +116,13 @@ Quadraticroots::Quadraticroots(pair a, pair b, pair c)
       roots=0;
   } else {
     roots=2;
-    if(b == 0.0) {
+    pair factor=0.5*b/a;
+    pair denom=b*factor;
+    if(denom == 0.0) {
       z1=Sqrt(-c/a);
       z2=-z1;
     } else {
-      pair factor=0.5*b/a;
-      z1=factor*sqrt1pxm1(-2.0*c/(b*factor));
+      z1=factor*sqrt1pxm1(-2.0*c/denom);
       z2=-z1-2.0*factor;
     }
   }
@@ -171,8 +178,9 @@ cubicroots::cubicroots(double a, double b, double c, double d)
     return;
   }
   
-  double ainv=1.0/a;
-  b *= ainv; c *= ainv; d *= ainv;
+  b /= a;
+  c /= a;
+  d /= a;
   
   double b2=b*b;
   double Q=3.0*c-b2;
@@ -338,10 +346,16 @@ path path::subpath(Int a, Int b) const
   }
 
   if (!cycles) {
-    if (a < 0)
+    if (a < 0) {
       a = 0;
-    if (b > n-1)
+      if(b < 0)
+        b = 0;
+    }
+    if (b > n-1) {
       b = n-1;
+      if(a > b)
+        a = b;
+    }
   }
 
   Int sn = b-a+1;
@@ -403,8 +417,8 @@ path path::subpath(double a, double b) const
     }   
     if (b > n-1) {
       b = n-1;
-      if (a > n-1)
-        a = n-1;
+      if (a > b)
+        a = b;
     }
     aL = nodes[(Int)floor(a)];
     aR = nodes[(Int)ceil(a)];
@@ -860,7 +874,8 @@ void lineintersections(std::vector<double>& T, const path& g,
 void intersections(std::vector<double>& S, std::vector<double>& T,
                    const path& g, const pair& p, const pair& q, double fuzz)
 {
-  if(q == p) {
+  double length2=(q-p).abs2();
+  if(length2 == 0.0) {
     std::vector<double> S1;
     intersections(S1,g,p,fuzz);
     size_t n=S1.size();
@@ -869,7 +884,7 @@ void intersections(std::vector<double>& S, std::vector<double>& T,
       T.push_back(0.0);
     }
   } else {
-    pair factor=(q-p)/((q-p).abs2());
+    pair factor=(q-p)/length2;
     std::vector<double> S1;
     lineintersections(S1,g,p,q,fuzz,true);
     size_t n=S1.size();
