@@ -36,7 +36,7 @@ public:
   nodelist nodes;
   
   picture() : labels(false), lastnumber(0), lastnumber3(0), T(identity),
-              transparency(false) {}
+              transparency(false), billboard(0) {}
   
   // Destroy all of the owned picture objects.
   ~picture();
@@ -56,6 +56,8 @@ public:
   
   bool havelabels();
   bool have3D();
+  bool havepng();
+  bool havenewpage();
 
   bbox bounds();
   bbox3 bounds3();
@@ -67,6 +69,9 @@ public:
     return transparency;
   }
   
+  int epstosvg(const string& epsname, const string& outname);
+  int pdftosvg(const string& pdfname, const string& outname);
+  
   int epstopdf(const string& epsname, const string& pdfname);
   int pdftoeps(const string& pdfname, const string& epsname);
   
@@ -74,13 +79,12 @@ public:
                   const string& prefix, const pair& bboxshift, bool svgformat); 
     
   bool postprocess(const string& prename, const string& outname, 
-                   const string& outputformat, double magnification,
-                   bool wait, bool view, bool pdftex, bool svgformat);
+                   const string& outputformat, bool wait, bool view,
+                   bool pdftex, bool epsformat, bool svg);
     
   // Ship the picture out to PostScript & TeX files.
   bool shipout(picture* preamble, const string& prefix,
-               const string& format, double magnification=0.0,
-               bool wait=false, bool view=true);
+               const string& format, bool wait=false, bool view=true);
  
   void render(GLUnurbs *nurb, double size2,
               const triple &Min, const triple& Max, double perspective,
