@@ -5,6 +5,10 @@
  * A place for useful utility functions.
  *****/
 
+#ifdef __CYGWIN__
+#define _POSIX_C_SOURCE 200809L
+#endif
+
 #include <cassert>
 #include <iostream>
 #include <cstdio>
@@ -184,7 +188,7 @@ string auxname(string filename, string suffix)
 {
   return buildname(filename,suffix,"_");
 }
-  
+
 sighandler_t Signal(int signum, sighandler_t handler)
 {
   struct sigaction action,oldaction;
